require 'uu/os/qos/qos_handler'
require 'uu/os/rest'
require 'uu/os/uesuri'
require 'uu/adk/app/app_create'
require 'uu/adk/app/app_attributes'
require 'uu/adk/app/app_set_attributes'
require 'uu/adk/app/app_get_entry_list'
require 'uu/adk/app/app_entry_attributes'



module UU
  module ADK
    # App service.
    # @deprecated Use {UU::OS::AppMetaModel} instead.
    module App
      extend self

      # Service path
      PATH = 'uu/adk/App'
      
      # Creates a new app . The command creates a new app  in a specified location. At the very least meta
      # artifact must be specified in {@link AppCreate} DTO. Competent role for the app  will be selected as
      # the most suitable according to specified container and meta artifact (executive/authorized role with connected interface), or can be
      # also specified in {@link AppCreate} DTO.
      #  
      # @param [String,UU::OS::UESURI] app_uri UESURI of the folder/org. unit/meta model where app  will be created.
      # @param [AppCreate] app DTO containing attributes of the new app 
      #
      # @return [UU::OS::UESURI] UESURI of the new app 
      def create(app_uri, app = nil)
        svc = UU::OS::REST::RemoteClient.new(App)
        payload = UU::ADK::App::AppCreate.new(app).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('create', app_uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

      #This command returns attributes representing the app  specified by the app_uri parameter. The command
      # does not change the state of the app  in the system.
      # 
      # @param [String,UU::OS::UESURI] app_uri UESURI of an app  whose attributes are returned
      #
      # @return [AppAttributes] DTO containing the app  attributes 
      def get_attributes(app_uri)
        svc = UU::OS::REST::RemoteClient.new(App)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getAttributes', app_uri)
          return App::AppAttributes.new(res)
        end
      end

      # Command for setting attributes of an app . The command can't modify references to objects like location, competent role
      # and so on.
      # 
      # @param [String,UU::OS::UESURI] app_uri UESURI of the app  whose attributes are to be changed
      # @param [AppSetAttributes] app DTO containing new attributes the app
      #
      # @return [UU::OS::UESURI] UESURI of the app 
      def set_attributes(app_uri, app = nil)
        svc = UU::OS::REST::RemoteClient.new(App)
        payload = UU::ADK::App::AppSetAttributes.new(app).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('setAttributes', app_uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

      # This command returns a list of entries (artifacts or shortcuts) enlisted in the specified app . Only entries listed
      # exactly in the specified app s are considered (so subordinate apps are not considered). The command does not
      # return all entries, only a sublist. The list of returned entries could be filtered by the name, code or type of an entry. The list is
      # sorted by the name (and codes in case are equal) of a entry by default, or it can be sorted arbitrarily by any of entry attributes
      # usable for the filtering or these attributes combinations. See the UES query documentation for more information.
      #
      # @param [String,UU::OS::UESURI] organizational_unit_uri UESURI of the app  from where entries (artifacts or shortcuts) are listed.
      # @param [AppGetEntryList] criteria DTO containing criteria for listing entries Available parameters are: {AppEntryAttributes#name
      # name}, {AppEntryAttributes#code code} and {AppEntryAttributes#type type}
      #
      # See {https://uu.unicornuniverse.eu/ues/sesm?&SessFree=ues:UNI-BT:UES.DOC%2FCMD_V1%2FUESQUERY UESQuery Documentation}.
      #
      # @return List of DTO representing entries of the app (artifacts or shortcuts).
      def get_entry_list(organizational_unit_uri, criteria = nil)
        svc = UU::OS::REST::RemoteClient.new(App)
        dto = App::AppGetEntryList.new(criteria)
        svc.add_parameter('pageIndex', dto.page_index)
        svc.add_parameter('pageSize', dto.page_size)
        svc.add_parameter('query', dto.query)
        svc.add_parameter('recursive', dto.recursive)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getEntryList', organizational_unit_uri)
          return UU::OS::REST::ResultList.new(App::AppGetEntryList, App::AppEntryAttributes, res)
        end
      end

    end
  end
end
